// This file is a part of the Framsticks GDK library.
// Copyright (C) 2002-2011  Szymon Ulatowski.  See LICENSE.txt for details.
// Refer to http://www.framsticks.com/ for further information.

#include "neuroclsobject.h"
#include "neurolibrary.h"
#include "paramvmobj.h"

#define FIELDSTRUCT NeuroClassExt
ParamEntry neuroclass_paramtab[]=
{
{"NeuroClass",1,10,"NeuroClass","The static NeuroClass object refers to class selected in the NeuroClassLibrary."},
{"name",0,0,"Class name","s",FIELD(name),},
{"longname",0,0,"Long, human readable name","s",FIELD(longname),},
{"description",0,0,"Long description","s",FIELD(description),},
{"prefinputs",0,0,"Preferred number of inputs","d",FIELD(prefinputs),},
{"prefoutput",0,0,"Provides output","d 0 1",FIELD(prefoutput),},
{"preflocation",0,0,"Preferred body location","d 0 2 ~None~Part~Joint",FIELD(preflocation),},
{"visualhints",0,0,"Visual hints","d",FIELD(visualhints),},
{"glyph",0,0,"Glyph vector data","s",GETSET(glyph),},
{"properties",0,0,"Interface object connected with neuron class properties","o Interface",GETONLY(props),},
{"summary",0,0,"Summary","s",GETONLY(summary),"Textual summary of all features",},
{0,0,0,},
}; 
#undef FIELDSTRUCT

static Param static_neuroclassparam(neuroclass_paramtab);

void NeuroClassExt::makeStaticObject(ExtValue *ret,NeuroClass *cl)
{
if (cl)
	*ret=ExtObject(&static_neuroclassparam,cl);
else
	ret->setEmpty();
}

#define FIELDSTRUCT NeuroClassLibObject
ParamEntry neuroclasslib_paramtab[]=
{
{"NeuroClassLibrary",1,4,"NeuroClassLibrary","Set of Neuron classes. You can access the selected class in the static NeuroClass object.",},
{"count",0,0,"class count","d",GETONLY(count),},
{"class",0,0,"current class","d",SETFIELD(current),"0 ... count-1"},
{"findClass",0,0,"select class by name","p(s class name)",PROCEDURE(p_findclass),},
{"getClass",0,0,"get class object by name","p NeuroClass(s class name)",PROCEDURE(p_getclass),},
{0,0,0,},
}; 
#undef FIELDSTRUCT

/////////////////////

NeuroClassLibObject::NeuroClassLibObject():
	param(neuroclasslib_paramtab,this),
	classparam(neuroclass_paramtab),
	current(0)
{
selectClass(-1);
}

void NeuroClassLibObject::selectClass(int i)
{
if (i >= lib().getClassCount()) i=-1;
if (current!=i)
	{
	current=i;
	classparam.select((i>=0)?lib().getClass(i):&invalid);
	}
}

int NeuroClassLibObject::set_current(PARAMSETARGS)
{
selectClass(arg->getInt());
return PSET_CHANGED;
}

void NeuroClassLibObject::p_findclass(PARAMPROCARGS)
{
selectClass(lib().findClassIndex(args->getString()));
}

void NeuroClassLibObject::p_getclass(PARAMPROCARGS)
{
int i=lib().findClassIndex(args->getString());
NeuroClassExt::makeStaticObject(ret,(i>=0)?lib().getClass(i):0);
}

int* NeuroClassLibObject::convertVectorData(const SString& s)
{
SList t;
int pos=0;
SString n;
while(s.getNextToken(pos,n,','))
	t+=(void*)atol((const char*)n);
int *ret;
if (!t.size()) return 0;
ret=new int[t.size()];
for (int i=0;i<t.size();i++)
	ret[i]=(long)t(i);
return ret;
}

int NeuroClassExt::set_glyph(PARAMSETARGS)
{
setSymbolGlyph(NeuroClassLibObject::convertVectorData(arg->getString()));
return PSET_CHANGED;
}

void NeuroClassExt::get_glyph(PARAMGETARGS)
{
SString txt;
if (vectordata)
	{
	int *d=vectordata;
	int len=*(d++);
	txt+=SString::valueOf(len);
	for (;len>0;len--)
		{
		txt+=',';
		txt+=SString::valueOf(*(d++));
		}
	}
ret->setString(txt);
}

void NeuroClassExt::get_props(PARAMGETARGS)
{
static Param par;
par=getProperties();
ret->setObject(ParamVMObj::makeObject(&par));
}
